!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

!latex \item Returns the magnetic field field line equations,
!latex $\dot\theta=B^\theta/B^\zeta$, $\dot s=B^s/B^\zeta$, and the tangent map, at an arbitrary point.

!latex \item The format of this subroutine is constrained by the NAG ode integration routines. 
!latex Sometimes it is required to have the full contravariant components of the field, 
!latex so the quantity \verb+Bzeta+$\equiv B^\zeta$ is returned through memory.

!latex \heading{logical control}

!latex \item If \verb+Nofe.eq.0+, then ${\bf B}$ is not continuous.

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

subroutine bf00aa( zeta , st , Bst )

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

  use constants, only : zero, half
  use numerical, only : vsmall
  use fileunits, only : ounit
  use inputlist, only : Wbf00aa, Nofe, Lradial
  use cputiming, only : Tbf00aa
  use allglobal, only : myid, cpus, mn, im, in, ll, iss, ss, Bzeta, Atmn, Azmn, Az, At, dss

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

  LOCALS

  REAL, intent(in)  :: zeta, st(6)
  REAL, intent(out) :: Bst(6)

  INTEGER           :: imn, ij, mm, ip, il
  REAL              :: lss, teta, arg, carg, sarg, dBu(3,0:2), TM(2,2), gBt, gBz, stz(1:3) , RpZ(1:3) , dR(0:3) , dZ(0:3) , jacobian , lgij(1:6)

  BEGIN(bf00aa)

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

  lss = min( max(st(1),iss(ll-1)) , iss(ll) ) ; teta = st(2) ! local variable used to ensure point is within bounds;

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

!latex \item A slow transform is performed, as the position $(s,\theta,\zeta)$ at which the field is required is arbitrary.
!latex It may be possible to speed this up by re-organizing the loops.

  dBu(1:3,0:2) = zero ! initialize summation;

  CALL(sa00aa,(lss)) ! interpolate Fourier harmonics of vector potential;

  do imn = 1 , mn ; arg = im(imn)*teta - in(imn)*zeta ; carg = cos(arg) ; sarg = sin(arg) ! sum Fourier modes;
   
   dBu(1,0) = dBu(1,0) + ( - im(imn)        * Azmn(0,imn) - in(imn)          * Atmn(0,imn) ) * sarg
   dBu(2,0) = dBu(2,0) - (                    Azmn(1,imn)                                  ) * carg
   dBu(3,0) = dBu(3,0) + (                                                     Atmn(1,imn) ) * carg
   
   dBu(1,1) = dBu(1,1) + ( - im(imn)        * Azmn(1,imn) - in(imn)          * Atmn(1,imn) ) * sarg
   dBu(2,1) = dBu(2,1) - (                    Azmn(2,imn)                                  ) * carg
   dBu(3,1) = dBu(3,1) + (                                                     Atmn(2,imn) ) * carg
   
   dBu(1,2) = dBu(1,2) + ( - im(imn)*im(imn) * Azmn(0,imn) - in(imn)*im(imn) * Atmn(0,imn) ) * carg
   dBu(2,2) = dBu(2,2) - ( -         im(imn) * Azmn(1,imn)                                 ) * sarg
   dBu(3,2) = dBu(3,2) + (                                 -         im(imn) * Atmn(1,imn) ) * sarg

  enddo ! end of do imn = 1 , mn;
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

!  if( ll.eq.1 .and. lss.lt.ss(ll)%s(1) .and. Nofe.eq.1 .and. Lradial.eq.2 ) then
!
!   stz(1:3) = (/ st(1) , st(2) , zeta /)
!
!   call co00aa( stz(1:3) , RpZ(1:3) , dR(0:3) , dZ(0:3) , jacobian , lgij(1:6) )
!
!  !write(ounit,'("bf00aa : gBz(0) = "es23.15" ; jacobian="es23.15" ; ")')At(ll,1,1)%s(0),jacobian
!
!   cput = MPI_WTIME()
!   
!   gBz = (6*lss*At(ll,1,0)%s(1))/dss(ll,2) - (6*lss**2*At(ll,1,0)%s(1))/(dss(ll,2)*dss(ll,1)) + At(ll,1,1)%s(0) -  & 
!  (4*lss*At(ll,1,1)%s(0))/dss(ll,1) + (3*lss**2*At(ll,1,1)%s(0))/dss(ll,2) - (2*lss*At(ll,1,1)%s(1))/dss(ll,1) +  & 
!  (3*lss**2*At(ll,1,1)%s(1))/dss(ll,2) + (15*lss**1.5*Cos(teta)*At(ll,2,0)%s(1))/(2.*dss(ll,2)) -  & 
!  (7*lss**2.5*Cos(teta)*At(ll,2,0)%s(1))/(dss(ll,2)*dss(ll,1)) + (3*Sqrt(lss)*Cos(teta)*At(ll,2,1)%s(0))/2. -  & 
!  (5*lss**1.5*Cos(teta)*At(ll,2,1)%s(0))/dss(ll,1) + (7*lss**2.5*Cos(teta)*At(ll,2,1)%s(0))/(2.*dss(ll,2)) -  & 
!  (5*lss**1.5*Cos(teta)*At(ll,2,1)%s(1))/(2.*dss(ll,1)) + (7*lss**2.5*Cos(teta)*At(ll,2,1)%s(1))/(2.*dss(ll,2)) +  & 
!  (9*lss**2*Cos(2*teta)*At(ll,3,0)%s(1))/dss(ll,2) - (8*lss**3*Cos(2*teta)*At(ll,3,0)%s(1))/(dss(ll,2)*dss(ll,1)) +  & 
!  2*lss*Cos(2*teta)*At(ll,3,1)%s(0) - (6*lss**2*Cos(2*teta)*At(ll,3,1)%s(0))/dss(ll,1) +  & 
!  (4*lss**3*Cos(2*teta)*At(ll,3,1)%s(0))/dss(ll,2) - (3*lss**2*Cos(2*teta)*At(ll,3,1)%s(1))/dss(ll,1) +  & 
!  (4*lss**3*Cos(2*teta)*At(ll,3,1)%s(1))/dss(ll,2) + (21*lss**2.5*Cos(3*teta)*At(ll,4,0)%s(1))/(2.*dss(ll,2)) -  & 
!  (9*lss**3.5*Cos(3*teta)*At(ll,4,0)%s(1))/(dss(ll,2)*dss(ll,1)) + (5*lss**1.5*Cos(3*teta)*At(ll,4,1)%s(0))/2. -  & 
!  (7*lss**2.5*Cos(3*teta)*At(ll,4,1)%s(0))/dss(ll,1) + (9*lss**3.5*Cos(3*teta)*At(ll,4,1)%s(0))/(2.*dss(ll,2)) -  & 
!  (7*lss**2.5*Cos(3*teta)*At(ll,4,1)%s(1))/(2.*dss(ll,1)) + (9*lss**3.5*Cos(3*teta)*At(ll,4,1)%s(1))/(2.*dss(ll,2))
!
!   gBt = &
!        (-6*lss*Az(ll,1,0)%s(1))/dss(ll,2) + (6*lss**2*Az(ll,1,0)%s(1))/(dss(ll,2)*dss(ll,1)) - Az(ll,1,1)%s(0) +  &
!  (4*lss*Az(ll,1,1)%s(0))/dss(ll,1) - (3*lss**2*Az(ll,1,1)%s(0))/dss(ll,2) + (2*lss*Az(ll,1,1)%s(1))/dss(ll,1) -  &
!  (3*lss**2*Az(ll,1,1)%s(1))/dss(ll,2) - (Cos(teta)*Az(ll,2,0)%s(0))/(2.*Sqrt(lss)) +  &
!  (15*lss**1.5*Cos(teta)*Az(ll,2,0)%s(0))/(2.*dss(ll,2)) - (7*lss**2.5*Cos(teta)*Az(ll,2,0)%s(0))/(dss(ll,2)*dss(ll,1)) -  &
!  (15*lss**1.5*Cos(teta)*Az(ll,2,0)%s(1))/(2.*dss(ll,2)) + (7*lss**2.5*Cos(teta)*Az(ll,2,0)%s(1))/(dss(ll,2)*dss(ll,1)) -  &
!  (3*Sqrt(lss)*Cos(teta)*Az(ll,2,1)%s(0))/2. + (5*lss**1.5*Cos(teta)*Az(ll,2,1)%s(0))/dss(ll,1) -  &
!  (7*lss**2.5*Cos(teta)*Az(ll,2,1)%s(0))/(2.*dss(ll,2)) + (5*lss**1.5*Cos(teta)*Az(ll,2,1)%s(1))/(2.*dss(ll,1)) -  &
!  (7*lss**2.5*Cos(teta)*Az(ll,2,1)%s(1))/(2.*dss(ll,2)) - Cos(2*teta)*Az(ll,3,0)%s(0) +  &
!  (9*lss**2*Cos(2*teta)*Az(ll,3,0)%s(0))/dss(ll,2) - (8*lss**3*Cos(2*teta)*Az(ll,3,0)%s(0))/(dss(ll,2)*dss(ll,1)) -  &
!  (9*lss**2*Cos(2*teta)*Az(ll,3,0)%s(1))/dss(ll,2) + (8*lss**3*Cos(2*teta)*Az(ll,3,0)%s(1))/(dss(ll,2)*dss(ll,1)) -  &
!  2*lss*Cos(2*teta)*Az(ll,3,1)%s(0) + (6*lss**2*Cos(2*teta)*Az(ll,3,1)%s(0))/dss(ll,1) -  &
!  (4*lss**3*Cos(2*teta)*Az(ll,3,1)%s(0))/dss(ll,2) + (3*lss**2*Cos(2*teta)*Az(ll,3,1)%s(1))/dss(ll,1) -  &
!  (4*lss**3*Cos(2*teta)*Az(ll,3,1)%s(1))/dss(ll,2) - (3*Sqrt(lss)*Cos(3*teta)*Az(ll,4,0)%s(0))/2. +  &
!  (21*lss**2.5*Cos(3*teta)*Az(ll,4,0)%s(0))/(2.*dss(ll,2)) - (9*lss**3.5*Cos(3*teta)*Az(ll,4,0)%s(0))/(dss(ll,2)*dss(ll,1)) -  &
!  (21*lss**2.5*Cos(3*teta)*Az(ll,4,0)%s(1))/(2.*dss(ll,2)) + (9*lss**3.5*Cos(3*teta)*Az(ll,4,0)%s(1))/(dss(ll,2)*dss(ll,1)) -  &
!  (5*lss**1.5*Cos(3*teta)*Az(ll,4,1)%s(0))/2. + (7*lss**2.5*Cos(3*teta)*Az(ll,4,1)%s(0))/dss(ll,1) -  &
!  (9*lss**3.5*Cos(3*teta)*Az(ll,4,1)%s(0))/(2.*dss(ll,2)) + (7*lss**2.5*Cos(3*teta)*Az(ll,4,1)%s(1))/(2.*dss(ll,1)) -  &
!  (9*lss**3.5*Cos(3*teta)*Az(ll,4,1)%s(1))/(2.*dss(ll,2))
!
!   write(ounit,'("bf00aa : lss="es23.15" sqg="es23.15" gBt="es23.15" err="es10.2" gBz="es23.15" err="es10.2" iota="es10.2" Bz="es23.15)')&
! lss,jacobian,dBu(2,0),dBu(2,0)-gBt,dBu(3,0),dBu(3,0)-gBz,gBt/gBz,gBz/jacobian
!
!  endif

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

  if( abs(dBu(3,0)).lt.vsmall ) then
   cput = MPI_WTIME() ; write(ounit,'("bf00aa : "f10.2" : ll="i3" ; zeta="es23.15" ; (s,t)=("es23.15" ,"es23.15" ) ; B^z="es23.15" ;")')cput-cpus,ll,zeta,st(1:2),dBu(3,0)
  endif

  FATAL(bf00aa,abs(dBu(3,0)).lt.vsmall)
  
  Bzeta = dBu(3,0) ! sometimes bf00aa is used to compute the field; WARNING MISSING JACOBIAN FACTOR?;

  Bst(1:2) = dBu(1:2,0) / dBu(3,0) ! normalize to toroidal field; toroidal angle zeta is the integration parameter;

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

!latex \item The tangent map satisfies 
!latex \be \frac{d}{d \phi} M = 
!latex \left(\begin{array}{cc}  \partial_s \dot s       &  \partial_\theta \dot s      \\
!latex                          \partial_s \dot \theta  &  \partial_\theta \dot \theta 
!latex         \end{array}
!latex \right)
!latex M. \ee

  TM(1,1) = ( dBu(1,1) - Bst(1) * dBu(3,1) ) / dBu(3,0) !  radial  derivative of  psi  dot;
  TM(1,2) = ( dBu(1,2) - Bst(1) * dBu(3,2) ) / dBu(3,0) ! poloidal derivative of  psi  dot;
  TM(2,1) = ( dBu(2,1) - Bst(2) * dBu(3,1) ) / dBu(3,0) !  radial  derivative of theta dot;
  TM(2,2) = ( dBu(2,2) - Bst(2) * dBu(3,2) ) / dBu(3,0) ! poloidal derivative of theta dot;

  Bst(3) = TM(1,1) * st(3) + TM(1,2) * st(5) ! tangent map;
  Bst(4) = TM(1,1) * st(4) + TM(1,2) * st(6) ! tangent map;
  Bst(5) = TM(2,1) * st(3) + TM(2,2) * st(5) ! tangent map;
  Bst(6) = TM(2,1) * st(4) + TM(2,2) * st(6) ! tangent map;

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

  if( Wbf00aa ) then ; cput = MPI_WTIME() ; write(ounit,'("bf00aa : "f10.2" : ll="i3" ; zeta="es23.15" ; Bst="6es23.15" ;")')cput-cpus,ll,zeta,Bst(1:6)
  endif

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

  RETURN(bf00aa)

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

end subroutine bf00aa

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

subroutine bf00aa_out(zeta,st)

  use inputlist, only : 
  use fileunits, only : ounit
  use allglobal, only : pi2nfp, ifb, iqfmin, jgd, kgd

  LOCALS

  REAL, intent(inout) :: zeta
  REAL, intent(in)    :: st(6)
  
  zeta = zeta + ifb*pi2nfp ! next intermediate output location;

!  if( iqfmin.eq.1 ) then
!
!   Ltr( 1:2 , jgd , kgd ) = (/ st(2) , st(1) /)
!   
!   write(ounit,'("bf00aa : Ltr(1:2,"i3","i3")="2f15.10)')jgd,kgd,Ltr( 1:2 , jgd , kgd )
!
!   kgd=kgd+1 ! QFmin surface;
!
!   pause
!
!  endif

  return

end subroutine bf00aa_out

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

REAL function bf00aa_end(zeta,st)

  use allglobal, only : pi2nfp, ifb

  LOCALS

  REAL, intent(inout) :: zeta
  REAL, intent(in)    :: st(6)
  
  bf00aa_end = zeta - ifb*pi2nfp ! integration termination;

  return

end FUNCTION bf00aa_end

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
