!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

!latex \item Compute eigenvalues / eigenvectors of derivative matrix,
!latex and construct derivative matrix using finite differences in series to compare with parallel construction provided by \verb+fc01aa+.

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

subroutine he01aa( Ndof , xorig )

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

  use constants, only : zero
  use numerical, only : sqrtmachprec
  use fileunits, only : ounit, hessianunit
  use inputlist, only : ext, Lcompare, Levalues, Levectors, Nvol, Whe01aa
  use cputiming
  use allglobal, only : myid, cpus, mn, im, in, pHess, dfd
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
  LOCALS
  
  INTEGER, intent(in)  :: Ndof
  REAL   , intent(in)  :: xorig(1:Ndof)
  
  LOGICAL              :: lderiv
  INTEGER              :: idof, jdof, iRZl, iRZ, imn, jRZl, jRZ, jmn, Ldvr, Ldvi, Lwk, LDA, iev, if02ebf
  REAL   , allocatable :: forig(:), xtry(:), ftry(:), aHess(:,:), er(:), ei(:), vr(:,:), vi(:,:), wk(:)
  REAL                 :: ijerr, ijden
  CHARACTER            :: JOB

  BEGIN(he01aa)

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

  ALLOCATE(forig,(1:Ndof))
  forig(1:Ndof)=zero

  if( myid.eq.0 ) then
   cput = MPI_WTIME() ; write(ounit,'("he01aa : "f10.2" : constructing derivative matrix;")')cput-cpus
  endif

  ADDCPU(he01aa)

  lderiv = .true.

  CALL(fc01aa,(Ndof,xorig(1:Ndof),forig(1:Ndof),lderiv)) ! construct Hessian in parallel;
  
  RESETCPU

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

  if( Lcompare ) then 

   if( myid.eq.0 ) then ; cput = MPI_WTIME() ; write(ounit,'("he01aa : "f10.2" : constructing derivative matrix in series ;")')cput-cpus
   endif
   
   ALLOCATE(xtry,(1:Ndof))
   ALLOCATE(ftry,(1:Ndof))
   
   ALLOCATE(aHess,(1:Ndof,1:Ndof))
   aHess(1:Ndof,1:Ndof) = zero
   
   lderiv = .false.
   
   ADDCPU(he01aa)

   do idof = 1,Ndof
    
    xtry(1:Ndof) = xorig(1:Ndof)

    xtry(idof) = xtry(idof) + dfd
    
    CALL(fc00aa,(Ndof,xtry(1:Ndof),ftry(1:Ndof),lderiv))
    
    aHess(1:Ndof,idof) =(  ftry(1:Ndof)-forig(1:Ndof) ) / dfd
    
   enddo
   
   RESETCPU

   if( myid.eq.0 ) then ! compare derivative matrix (in intuitive format);
    
    do iRZl = 2 , Nvol-1 ! loop over interfaces;
     do iRZ = 0,1 ! loop over RZ;
      do imn = 1,mn ! loop over Fourier harmonics;
       
       if( iRZ.eq.1 .and. imn.eq.1 ) cycle ! nothing depends on this;
       
       idof = ( iRZl - 2 ) * ( mn + mn - 1 ) + iRZ*mn + imn - iRZ ! label degree of freedom;
       
       do jRZl = 2 , Nvol-1 ! loop over interfaces;
        do jRZ = 0,1 ! loop over RZ;
         do jmn = 1,mn ! loop over Fourier harmonics;
          
          if( jRZ.eq.1 .and. jmn.eq.1 ) cycle ! nothing depends on this;
          
          jdof = ( jRZl - 2 ) * ( mn + mn - 1 ) + jRZ*mn + jmn - jRZ ! label degree of freedom;
          
          FATAL(he01aa, idof.le.0 .or. idof.gt.Ndof)
          FATAL(he01aa, jdof.le.0 .or. jdof.gt.Ndof)
          
          ijerr = abs( aHess(idof,jdof) - pHess(idof,jdof) ) ; ijden = max( abs(aHess(idof,jdof)) , sqrtmachprec )
          
          write(ounit,'("ll="i2" ; iRZ="i2" ; imn="i4" (m,n)=("i3" ,"i3" ) ; idof="i6" : ll="i2" ; jRZ="i2" ; jmn="i4" (m,n)=("i3" ,"i3" ) ; jdof="i6" : aH,pH="2es23.15" err="es12.5" ,"f13.6"% ;")')&
        iRZl, iRZ, imn, im(imn), in(imn), idof, jRZl, jRZ, jmn, im(jmn), in(jmn), jdof, aHess(idof,jdof), pHess(idof,jdof), ijerr, 100 * ijerr / ijden
          
         enddo
        enddo
       enddo
       
      enddo
     enddo
    enddo
    
   endif ! end of if( myid.eq.0 ) 
   
   DEALLOCATE(ftry)
   DEALLOCATE(xtry)
   
   DEALLOCATE(aHess)
   
  endif ! end of if( Lcompare )
  
  DEALLOCATE(forig)
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

  if( myid.eq.0 ) then ! only this node will construct eigenvalues/eigenvectors ;
   
   if( Levalues .or. Levectors ) then

   if( myid.eq.0 ) then
    cput = MPI_WTIME() ; write(ounit,'("he01aa : "f10.2" : constructing eigenvalues/vectors of derivative matrix; Levalues,Levectors="2L2)')cput-cpus,Levalues,Levectors
   endif
    
    Lwk = 4*Ndof ; LDA = Ndof

    if( Levectors ) then ; JOB='V' ; Ldvr = Ndof ; Ldvi = Ndof
    else                 ; JOB='N' ; Ldvr =  1   ; Ldvi =  1
    endif
    
    ALLOCATE(er,(1:Ndof))        ! real eigenvalues;
    ALLOCATE(ei,(1:Ndof))        ! imag eigenvalues;
    ALLOCATE(vr,(1:Ldvr,1:Ndof)) ! real eigenvectors;
    ALLOCATE(vi,(1:Ldvi,1:Ndof)) ! real eigenvectors;
    ALLOCATE(wk,(1:Lwk))         ! workspace;
    
   !ALLOCATE(aHess,(1:Ndof,1:Ndof))
    
   !aHess(1:Ndof,1:Ndof) = pHess(1:Ndof,1:Ndof)
    
    if02ebf=0 ; call F02EBF( JOB , Ndof , pHess(1:Ndof,1:Ndof) , LDA , er(1:Ndof) , ei(1:Ndof) , vr(1:Ldvr,1:Ndof) , Ldvr , vi(1:Ldvi,1:Ndof) , Ldvi , wk(1:Lwk) , Lwk , if02ebf )
    
    if( Whe01aa ) then 
     cput = MPI_WTIME()
     do iev =1,Ndof ; write(ounit,'("he01aa : "f10.2" : derivative matrix evalues = ("es23.15" ,"es23.15" ) ;")')cput-cpus,er(iev),ei(iev)
     enddo
    endif
    
!   er = sqrt(er**2+ei**2) ; ifail=0 ; N1=1 ; ORDER='A' ; call M01CAF( er , N1 , Ndof , ORDER , ifail )
!   write(ounit,'("jk00aa : "f10.2" : (normalized) evalues = "11es9.2" . . ")')cput-cpus,er(1:min(11,Ndof))/maxval(er)

    open(hessianunit,file="."//trim(ext)//".Hev",status="unknown",form="unformatted") ! this will contain Poincare plot data;
    write(hessianunit)Ndof,Ldvr,Ldvi
    write(hessianunit)er
    write(hessianunit)ei
    write(hessianunit)vr
    write(hessianunit)vi
    close(hessianunit)

    DEALLOCATE(er)
    DEALLOCATE(ei)
    DEALLOCATE(vr)
    DEALLOCATE(vi)
    DEALLOCATE(wk)
   !DEALLOCATE(aHess)
    
   endif ! end of if( Lvalues .or. Levectors )
   
  endif ! end of if( myid.eq.0 ) 

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
  RETURN(he01aa)
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
end subroutine he01aa

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

