!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

!latex \item Calls NAG routine to find zero of force-balance using function values only.

!latex \item This routine is called from \verb+spec+, and only if \verb+Lc05ndf.eq.true+.
!latex \item The NAG routine \verb+C05NDF+ is used to find a zero of the force-balance vector, as computed by \verb+fc00aa+.
!latex \item The iterations are terminated if $|f|<$\verb+forcetol+.

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

subroutine jk01aa( lNgdof , xtry , ic05ndf )
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
  use constants, only : zero, one, two
  use numerical, only : machprec, sqrtmachprec
  use fileunits, only : ounit, iterationunit
  use inputlist, only : Wjk01aa, ext, Nvol, forcetol, factor, forceerr
  use cputiming
  use allglobal, only : writin, numprocs, myid, cpus, &
                        mn, Rbc, Zbs, & ! required for unpacking geometry vector in preparation for call to writin;
                        mBomn, mIomn ! these are returned by fc01aa;

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
  LOCALS

  INTEGER, intent(in)    :: lNgdof
  REAL   , intent(inout) :: xtry(1:lNgdof)
  INTEGER, intent(out)   :: ic05ndf

  LOGICAL :: lderiv
  INTEGER :: irevcm, ml, mu, mode, LDFJAC, LR, its(1:2), ll
  REAL    :: xtol, epsfcn, mdf, lfactor
  
  REAL    :: fvec(1:lNgdof), diag(1:lNgdof), fjac(1:lNgdof,1:lNgdof), RR(1:lNgdof*(lNgdof+1)/2), QTF(1:lNgdof), WK(1:lNgdof,1:4)

  BEGIN(jk01aa) ! this sets cpui & cpuo

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

  xtol = forcetol ; epsfcn = sqrtmachprec ; ic05ndf = 1 ! set some accuracy parameters;
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
  LDFJAC = lNgdof ; LR = lNgdof * (lNgdof+1) / 2 ; ml = lNgdof-1 ; mu = lNgdof-1 ! local variables, array dimensions;

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
  mode = 0 ! no scaling factors; diag is computed internally; if mode=2 then scaling factors (diag) needs to be provided;
  
  lfactor = factor ! used to determine initial step bound;
  
  lderiv = .false. ! derivatives (computed in fc00aa) are not required;
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
  fvec(1:lNgdof)=zero
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

  its(1:2) = 0
  
  if( Wjk01aa ) then ; cput = MPI_WTIME() ; write(ounit,'("jk01aa : "f10.2" : myid="i3" ; calling fc00aa ;")')cput-cpus,myid
  endif

  CALL(fc00aa,(lNgdof,xtry(1:lNgdof),fvec(1:lNgdof),lderiv)) ! on initial entry to C05NDF, fvec is required;
  
  if( Wjk01aa ) then ; cput = MPI_WTIME() ; write(ounit,'("jk01aa : "f10.2" : myid="i3" ; called  fc00aa ;")')cput-cpus,myid
  endif
  
  if( myid.eq.0 ) then
   cput = MPI_WTIME()
   mdf = sqrt( sum(fvec**2) / lNgdof )
   write(ounit,3011)cput-cpus,its(1:2),forceerr,mdf,cput-cpuo,(mBomn(ll),mIomn(ll),ll=1,Nvol-1)
   cpuo=cput
  !CALL(writin) ! write restart file; save geometry to ext.end; Rbc, Zbs consistent with xtry;
  endif

  if( forceerr.lt.10*forcetol ) then ; ic05ndf = 0 ; goto 9999 ! force-balance is satisfied;
  endif
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

  diag(1:lNgdof) = one ; fjac(1:LDFJAC,1:lNgdof) = zero ; RR(1:LR) = zero ; QTF(1:lNgdof) = zero ; WK(1:lNgdof,1:4) = zero
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
  irevcm = 0 ; ic05ndf = 1 ! required for initial entry; herefater unchanged by user;
  
  if( myid.eq.0 ) open(iterationunit,file="."//trim(ext)//".fits",status="unknown")

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
 
  if( Wjk01aa ) then ; cput = MPI_WTIME() ; write(ounit,'("jk01aa : "f10.2" : myid="i3" ; entering reverse communication loop ;")')cput-cpus,myid
  endif

  if( myid.eq.0 ) then
   write(ounit,'("jk01aa : "f10.2" : ")')cput-cpus
   write(ounit,'("jk01aa : "f10.2" : estimated time = ",f10.0"s = "f8.1"m = "f6.1"h = "f5.1"d ;")')cput-cpus,lNgdof*(cput-cpui)/(/1,60,60*60,60*60*24/)
   write(ounit,'("jk01aa : "f10.2" : ")')cput-cpus
  endif

  do ! C05NDF uses reverse communication;

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

   call C05NDF(irevcm,lNgdof,xtry(1:lNgdof),fvec(1:lNgdof),xtol,ml,mu,epsfcn,diag,mode,lfactor,fjac(1:LDFJAC,1:lNgdof),LDFJAC,RR(1:LR),LR,QTF(1:lNgdof),WK(1:lNgdof,1:4),ic05ndf)
   
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

   select case(irevcm)

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
    
   case(0) ! final exit; C05NDF has terminated;

    if( myid.eq.0 ) then
     cput = MPI_WTIME()
     select case(ic05ndf)
     case(0) ; write(ounit,'("jk01aa : "f10.2" ; success ;")')cput-cpus
     case(1) ; write(ounit,'("jk01aa : "f10.2" ; C05NDF ic05ndf="i3" ; input error ;")')                                      cput-cpus,ic05ndf
     case(2) ; write(ounit,'("jk01aa : "f10.2" ; C05NDF ic05ndf="i3" ; irevcm error ;")')                                     cput-cpus,ic05ndf
     case(3) ; write(ounit,'("jk01aa : "f10.2" ; C05NDF ic05ndf="i3" ; no further improvement possible ; xtol too small ;")') cput-cpus,ic05ndf
     case(4) ; write(ounit,'("jk01aa : "f10.2" ; C05NDF ic05ndf="i3" ; not making good progress ;")')                         cput-cpus,ic05ndf
     case(5) ; write(ounit,'("jk01aa : "f10.2" ; C05NDF ic05ndf="i3" ; not making good progress ;")')                         cput-cpus,ic05ndf
     case default
      FATAL(jk01aa : illegal C05NDF ic05ndf : C05NDF error,.true.)
     end select
    endif
    
    exit ! escape from infinite do loop;

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
    
   case(1) ! indicates start of new iteration; no action is required; xtry and fvec available for printing; fvec must not be changed;
    
    its(1) = its(1) + 1
    
    do ll = 1,Nvol-1 ; Rbc(ll,1:mn) = xtry((ll-1)*(mn+mn-1)+1   :(ll-1)*(mn+mn-1)+mn     ) ! unpack coordinate degrees-of-freedom vector;
                       Zbs(ll,2:mn) = xtry((ll-1)*(mn+mn-1)+mn+1:(ll-1)*(mn+mn-1)+mn+mn-1)
    enddo
    
    if( myid.eq.0 ) then
     cput = MPI_WTIME()
     mdf = sqrt( sum(fvec**2) / lNgdof )
     write(ounit,3011)cput-cpus,its(1:2),forceerr,mdf,cput-cpuo,(mBomn(ll),mIomn(ll),ll=1,Nvol-1)
     cpuo=cput
     CALL(writin) ! write restart file; save geometry to ext.end; Rbc, Zbs consistent with xtry;
     write(iterationunit,'(2i9,2es23.15)')its(1:2),forceerr,mdf
     call flush(iterationunit)
    endif
    
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

   case(2) ! before re-entry to C05NDF, fvec must contain the function values;
    
    its(2) = its(2) + 1
  
    if( Wjk01aa ) then ; cput = MPI_WTIME() ; write(ounit,'("jk01aa : "f10.2" : myid="i3" ; calling fc00aa ;")')cput-cpus,myid
    endif
    
    CALL(fc00aa,(lNgdof,xtry(1:lNgdof),fvec(1:lNgdof),lderiv))
    
    if( Wjk01aa ) then ; cput = MPI_WTIME() ; write(ounit,'("jk01aa : "f10.2" : myid="i3" ; called  fc00aa ;")')cput-cpus,myid
    endif

   !if( Wjk01aa .and. myid.eq.0 ) then ; cput = MPI_WTIME() ; write(ounit,'("jk01aa : "f10.2" : myid="i3" ; its="2i6)')cput-cpus,myid,its(1:2)
   !endif

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

   case default
    
    FATAL(jk01aa : illegal irevcm : C05NDF error,.true.)

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
    
   end select

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
   
  enddo ! end of infinite do loop;

  if( Wjk01aa ) then ; cput = MPI_WTIME() ; write(ounit,'("jk01aa : "f10.2" : myid="i3" ; finished reverse communication loop ;")')cput-cpus,myid
  endif

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

  if( myid.eq.0 ) close(iterationunit)

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
  RETURN(jk01aa)
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

3011 format("jk01aa : "f10.2" : "i3,i7" ; |f|="2es14.7" ; time="f8.1"s ; |BB|,|I|="999(f6.2,f5.1)" ;")

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

end subroutine jk01aa

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
