!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

!latex \item Constructs spectrally-condensed Fourier representation of interfaces using a stream function.
!latex The spectral-width minimizing angle is calculated using a Newton method.
!latex This routine is only called at the start of the calculation, and only if \verb+Iswmin.eq.1+, 
!latex to convert the given Fourier representation of the boundary to a spectrally condensed representation.

!latex \item The geometry of the initial interfaces is given
!latex \be R(\theta,\zeta)=\sum_i R_i \cos(m_i \theta - n_i \zeta) , \,\,\, Z(\theta,\zeta)=\sum_i Z_i \sin(m_i \theta - n_i \zeta) .\ee

!latex \item A new angle shall be introduced via a stream function, $\lambda$, according to 
!latex \be \theta &=&\bar \theta + \sum_j \lambda_j \sin(m_j \bar \theta - n_j \zeta). \ee

!latex \item The Fourier harmonics in the new angle are
!latex \be \bar R_k = \oint\oint R(\theta,\zeta) \cos(m_k \bar \theta - n_k \zeta) d\bar \theta d \zeta, \,\,\,
!latex     \bar Z_k = \oint\oint Z(\theta,\zeta) \sin(m_k \bar \theta - n_k \zeta) d\bar \theta d \zeta. \ee

!latex \item The spectral-width in the new angle is defined
!latex \be M = \frac{1}{2}\sum_k (m_k^p+n_k^q) \left(\bar R_k^2 + \bar Z_k^2\right),\ee
!latex where $m_k^p = 0$ for $m_k=0$ and $n_k^q = 0$ for $n_k=0$, and where $p\equiv$\verb+pwidth+ and $q\equiv$\verb+qwidth+ are given on input.

!latex \item The variation in spectral-width due to variations, $\delta \lambda_j$ is
!latex \be \frac{\partial M}{\partial \lambda_j} = 
!latex     \sum_k ({\bar m}_k^p+{\bar n}^q)\left( \bar R_k\frac{\partial \bar R_k}{\partial \lambda_j} + \bar Z_k\frac{\partial \bar Z_k}{\partial \lambda_j} \right). 
!latex \ee

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

module spectralm
  INTEGER           :: nc05nbf      ! iteration counter;
  REAL              :: offset = 1.0 ! to move solution away from zero;
  REAL, allocatable :: dRjkdl(:,:), dZjkdl(:,:), dRmndl(:,:), dZmndl(:,:)
end module spectralm

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

subroutine sw00ac
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
  use constants, only : zero
  use numerical, only : machprec
  use fileunits, only : ounit
  use inputlist, only : Wsw00ac
  use cputiming, only : Tsw00ac
  use allglobal, only : myid, cpus, mn, im, in, ll, Rbc, Zbs, Ntz

  use spectralm
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
  LOCALS
  
  INTEGER           :: ic05nbf, Nlambda, Lwk
  REAL              :: xtol, dR, dZ
  REAL, allocatable :: lambda(:), ldMdl(:), wk(:)
  
  external :: GldMdl ! derivatives of spectral-width wrt Fourier harmonics of angle transformation;
  
  BEGIN(sw00ac)
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
  Nlambda = mn-1 ! #degrees-of-freedom;

  Lwk = Nlambda * ( 3 * Nlambda + 13 ) / 2
  
  ALLOCATE(lambda,(2:mn)) ! independent variables;
  ALLOCATE(ldMdl,(2:mn)) ! derivatives of spectral-width wrt independent variables;

  ALLOCATE(wk,(Lwk)) ! workspace required for NAG routine C05NBF;
  
  ic05nbf = 1 ; lambda(2:mn) = offset ; xtol = machprec ! initial guess for lambda harmonics is offset from zero; note that this offset must be accomodated below in GldMdl;
  
  nc05nbf = 0 ! local iteration counter;

  ALLOCATE(dRjkdl,(1:Ntz,1:mn))
  ALLOCATE(dZjkdl,(1:Ntz,1:mn))

  ALLOCATE(dRmndl,(1:mn,1:mn))
  ALLOCATE(dZmndl,(1:mn,1:mn))

!latex \item This routine seeks a zero of a function, ${\bf F}_j\equiv\partial M/\partial \lambda_j$, 
!latex where the degress of freedom are the $\lambda_j$, using the NAG routine \verb+c05nbf+.
!latex (This routine uses function values only, and with a little effort the derivatives could be calculated and more efficient ?? routines enabled.)

  CALL(C05NBF,( GldMdl , Nlambda, lambda(2:mn) , ldMdl(2:mn) , xtol , wk(1:Lwk) , Lwk , ic05nbf )) ! solve non-linear equations;
  
  DEALLOCATE(dRjkdl)
  DEALLOCATE(dZjkdl)

  DEALLOCATE(wk)
  
  cput = MPI_WTIME()   
  dR = sqrt( sum(( Rbc(ll,1:mn)-dRmndl(1:mn,1) )**2) /  mn    ) ! this measures how much the Fourier representation has changed from the initial;
  dZ = sqrt( sum(( Zbs(ll,2:mn)-dZmndl(2:mn,1) )**2) / (mn-1) )
  write(ounit,1002)cput-cpus,myid,ll,ic05nbf,mn,nc05nbf,sqrt(sum(ldMdl**2)/Nlambda),dR,dZ,cput-cpui
1002 format("sw00ac : "f10.2" : myid="i3" ; ll="i3" ; ic05nbf="i2" ; mn="i4" ; nc05nbf="i4" ; |dMdl|="es9.2" ; dR,dZ="2es9.2" ; time="f10.2"s ;")

  DEALLOCATE(lambda)
  DEALLOCATE(ldMdl)

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

  Rbc(ll,1:mn) = dRmndl(1:mn,1) ! note that the Fourier representations are calculated in GldMdl and are passed through global using dummy "workspace" arrays;
  Zbs(ll,2:mn) = dZmndl(2:mn,1)

  DEALLOCATE(dRmndl)
  DEALLOCATE(dZmndl)

  if( Wsw00ac ) then
   cput = MPI_WTIME()
   write(ounit,'("sw00ac : "f10.2" : ll="i3" ; Rbc="99f15.11)')cput-cpus,ll,Rbc(ll,1:mn)
   write(ounit,'("sw00ac : "f10.2" : ll="i3" ; Zbs="99f15.11)')cput-cpus,ll,Rbc(ll,1:mn)
  endif

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

  RETURN(sw00ac)
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
end subroutine sw00ac

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!


!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

subroutine GldMdl( Nlambda , lambda , ldMdl , iflag )
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
  use constants, only : zero, pi2
  use numerical, only :
  use fileunits, only : ounit
  use inputlist, only : Nfp, Ntor, pwidth, qwidth
  use allglobal, only : myid, cpus, pi2nfp, ll, mn, im, in, Rbc, Zbs, Nt, Nz, Ntz, isr, trigm, trign, trigwk, efmn, ofmn, cfmn, sfmn, ijreal, ijimag, jireal, jiimag
  use spectralm
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
  LOCALS
  
  INTEGER, intent(in)    :: Nlambda
  REAL   , intent(in)    :: lambda(2:Nlambda+1)
  REAL   , intent(out)   :: ldMdl(2:Nlambda+1)
  INTEGER, intent(inout) :: iflag
  
  INTEGER                :: imn, jmn, kmn, jj, kk, jk, ifail, mmpp, nnqq
  REAL                   :: tb, tt, zz, arg, ca, sa, barg, sba
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
  FATAL(sw00ac,Nlambda.ne.mn-1)
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
! construct lambda on regular \bar\theta grid;
  ofmn(1:mn) = (/ zero , lambda(2:mn) - offset /) ; efmn(1:mn) = zero ; cfmn(1:mn) = zero ; sfmn(1:mn) = zero 
  
  call invfft( mn , im(1:mn) , in(1:mn) , efmn(1:mn) , ofmn(1:mn) , cfmn(1:mn) , sfmn(1:mn) , Nt , Nz , ijreal(1:Ntz) , ijimag(1:Ntz) , isr , trigm , trign , trigwk )
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
! construct R, Z (and their derivatives) on regularly \bar\theta grid, which is an irregular \theta grid and so must be solved using a slow transform;
  
  dRjkdl(1:Ntz,1:mn) = zero
  dZjkdl(1:Ntz,1:mn) = zero
  
  do kk = 0,Nz-1 ; zz = kk * pi2nfp / Nz
   do jj = 0,Nt-1 ; jk = 1 + jj + kk*Nt ; tb = jj * pi2 / Nt ; tt = tb + ijreal(jk)
    
    do imn = 1,mn ; arg = im(imn)*tt - in(imn)*zz ; ca=cos(arg) ; sa=sin(arg) ! slow transform as tt is not regular;
     
     dRjkdl(jk, 1 ) = dRjkdl(jk, 1 ) + Rbc(ll,imn) * ca ! R;
     dZjkdl(jk, 1 ) = dZjkdl(jk, 1 ) + Zbs(ll,imn) * sa ! Z;
     
     do jmn = 2,mn ; barg = im(jmn)*tb - in(jmn)*zz ; sba=sin(barg) ! NOTE: lambda MAY be higher resolution, though it seems to be not implemented at present;
      
!latex \item Differentiating the Fourier harmonic wrt $\lambda_j$ is equivalent to Fourier decomposing the derivative:
!latex \be \frac{\partial \bar R_k}{\partial \lambda_j} \equiv \left(\frac{\partial \bar R}{\partial \lambda_j}\right)_k , 
!latex     \frac{\partial \bar Z_k}{\partial \lambda_j} \equiv \left(\frac{\partial \bar Z}{\partial \lambda_j}\right)_k . \ee
      
      dRjkdl(jk,jmn) = dRjkdl(jk,jmn) - Rbc(ll,imn) * sa * im(imn) * sba ! d R / d l_j
      dZjkdl(jk,jmn) = dZjkdl(jk,jmn) + Zbs(ll,imn) * ca * im(imn) * sba ! d Z / d l_j
      
     enddo ! end of do jmn = 2,mn
     
    enddo ! end of do imn = 1,mn
    
   enddo ! end of do jj = 0,Nt-1
  enddo ! end of do kk = 0 Nz-1
  
! now need to combine this information to get derivatives of spectral-width wrt lambda harmonics; these Fourier decompositions are reduced resolution;
  
  do jmn = 1,mn
   call tfft( Nt,Nz,dRjkdl(1:Ntz,jmn),dZjkdl(1:Ntz,jmn),isr,trigm,trign,trigwk,mn,im(1:mn),in(1:mn),dRmndl(1:mn,jmn),ofmn(1:mn),cfmn(1:mn),dZmndl(1:mn,jmn),ifail )
  enddo
  
  ldMdl(2:mn) = zero
  
  do jmn = 2,mn ! loop over non-trivial lambda harmonics;
   
   do kmn = 2,mn ; mmpp = 0 ; nnqq = 0 ! recall that im(1)=0 and in(1)=0;
     
    if( im(kmn).ne.0 .and. pwidth.gt.0 ) mmpp = abs(im(kmn)    )**pwidth
    if( in(kmn).ne.0 .and. qwidth.gt.0 ) nnqq = abs(in(kmn)/Nfp)**qwidth
    
    ldMdl(jmn) = ldMdl(jmn) + ( mmpp + nnqq ) * ( dRmndl(kmn,1) * dRmndl(kmn,jmn) + dZmndl(kmn,1) * dZmndl(kmn,jmn) ) ! derivatives of spectral-width wrt m.ne.0 lambda harmonics;
    
   enddo
   
  enddo
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
  efmn(1:mn) = dRmndl(1:mn,1) ! pass these back through common;
  ofmn(1:mn) = dZmndl(1:mn,1) ! pass these back through common;

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

  nc05nbf = nc05nbf + 1

  return
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
end subroutine GldMdl

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
